# Purpose: Create a cleartool command script to attach attribute values
#   for based on an input file 
# Input: A tab delimited file with the first line defining
#   the column names.
#   Column 1: level, ignored
#   Column 2: element name
#   Column 2+: attribute names
#   All lines following contain values for one element
#   Values may be blank.
#
# Output: A file with ClearCase commands to attach attributes
#   to the element.
#   Example:
#     cleartool mkattr -rep DL1 "\"Y\"" element_a@@
#     cleartool mkattr -rep ENVIR "\"B\"" element_a@@
#     cleartool mkattr -rep LANG "\"COBOL\"" element_a@@
#     cleartool mkattr -rep CICS "\"Y\"" element_b@@
#     cleartool mkattr -rep -def DBOTHR element_b@@
#
# Date: 2006-06-22
# Author: Anthony Lee
#
# (c) Copyright IBM Corp. 2008
# 
# To do:
# 1. attribute enumeration values definitions
# 2. more error checking
#
# Usage:
#   You must specify the input and output files with the
#   -i and -o parameter.
#   Example:
#     ccperl ____.pl -i infile -o outfile
#   -h for usage synopsis
#   -v for verbose diagnostics
#   -t for using tab-delimited input files (default)
#   -c for using comma-delimited input files
#   -m for creating also a file called <output_file>.cmd to
#      create the attribute types
#   -e for checking element exists before output


# use strict;
use Cwd;
use Getopt::Std;

my $usage = "test [-h] [-v] [-c] [-m] [-e] [-a <attribute_definitions>] -i <input_file> -o <output_file>";

if (!getopts('hvcmea:i:o:') || $opt_h) {
	print "$usage\n";
	exit(0);
}

my $verbose = $opt_v;
my $tab_delimited = $opt_t;
my $comma_delimited = $opt_c;
my $infile = $opt_i;
my $outfile = $opt_o;
my $check_exist = $opt_e;

my $delimiter = "\t"; # tab by default 

my $mkattr_cmd = "cleartool mkattr -rep ";
my $at_at = "@@\n";
my $outtype = $outfile . "1"; 

if ($verbose) {
	print "in file: $infile \n";
	print "out file: $outfile \n";
}
if ($tab_delimited) {
	$delimiter = "\t";
}
if ($comma_delimited) {
	$delimiter = ",";
}

open (INPUT, "<$infile") || die "Cannot open file $opt_i : $! \n";
open (OUTPUT, ">$outfile.cmd") || die "Cannot open file $opt_o : $! \n";

# read the first line of column names and assign attribute names
my $ignore, @attr_names;
$_ = <INPUT>;
chomp($_);
($ignore,$ignore,@attr_names) = split(/$delimiter/, $_);
if ($verbose) {
	my $attr_names_ref = \@attr_names;
	my $name;
	for $name (@$attr_names_ref) {
	     print "Attribute Name: $name \n";
	}
}

# find and remember the LANG index
my $attr_names_ref = \@attr_names;
my $name;
my $lang_index = 0;
for $name (@$attr_names_ref) {
	if ($name eq "LANG") {
		last;
	} else {
		$lang_index++;
	}
}
print "lang index: $lang_index\n" if $verbose;


# output command to create attribute types
if ($opt_m) {
	open (MKTYPE, ">$outtype.cmd") || die "Cannot open file $opt_o.cmd : $! \n";
	my $attr_names_ref = \@attr_names;
	my $name;
	for $name (@$attr_names_ref) {
	     print MKTYPE "cleartool mkattype -nc -vtype string -def \\\"N\\\" $name \n";
	}
	print MKTYPE "# Remember to add description and validate -vtype and -default values";
	close(MKTYPE);
}

# read the rest of the files with element name and attribute values
my $ignore, $element, @attributes;
my $index, $attr_value;

while (<INPUT>) {
	chomp;
	# parse input line into element name and attribute values
	($ignore,$element,@attributes) = split(/$delimiter/);
	print "element:",$element,$at_at if ($verbose);

	my $elem_ext;

	# determine LANG type first
	my $elem_lang = @attributes[$lang_index];
	print "element LANG is: $elem_lang\n" if $verbose;
	if ($elem_lang eq "COBOL2") {
		$elem_ext = "CBL";
	} elsif ($elem_lang eq "TELON") {
		$elem_ext = "TLN";
	} elsif ($elem_lang eq "SAS") {
		print "Ignoring SAS file: $element\n";
		next;
	} else {
		print "Unknown LANG type: $elem_lang, element: $element\n" if $verbose;
		next;
	}
	my $full_elem = "$element.$elem_ext";
	print "full element name is: $full_elem\n" if $verbose;

	# don't generate cmd if the file does not exist 
	if ($check_exist && !(-e $full_elem)) {
		print "Element $full_elem does not exist.\n";
		next;
	}

	# output 1 cleartool cmd to mkattr for each attribute
	for ($index = 0; $index <= $#attr_names; $index) {
		$attr_value = @attributes[$index];
		print "\t@attr_names[$index]: $attr_value \n" if ($verbose);
		# print cleartool make attribute command
		if ($attr_value eq "") {
			# no value, use -default
			print OUTPUT $mkattr_cmd, "-def ", @attr_names[$index], " ", $element, ".", $elem_ext, $at_at;
		} else {
			# use value specified
			print OUTPUT $mkattr_cmd, @attr_names[$index], ' "\\"', $attr_value, '\\"" ', $element, ".", $elem_ext, $at_at;
		}
		$index++;
	}
}
close(INPUT);
close(OUTPUT);
